/*
Lexmark 1100 printer driver for Linux
Copyright (C) 1999   Tim Engler

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "ErrorDiffusionAlg.h"
#include "math.h"
#include "RGB.h"
#include <stdio.h>
#define JARVIS_FILTER

#ifdef JARVIS_FILTER

#define FILTER_HEIGHT 3
#define FILTER_WIDTH 5
#define DIVIDER 48

int errorMatrix[FILTER_HEIGHT][FILTER_WIDTH] = {
	{
		1, 3, 5, 3, 1
	},
	{
		3, 5, 7, 5, 3
	},
	{
		5, 7, 0, 0, 0
	}
};
#endif

#ifdef FLOYD_FILTER

#define FILTER_HEIGHT 3
#define FILTER_WIDTH 3
#define DIVIDER 16

int errorMatrix[FILTER_HEIGHT][FILTER_WIDTH] = {
	{
		0, 0, 0
	},
	{
		1, 5, 3
	},
	{
		7, 0, 0
	}
};
#endif

ErrorDiffusionAlg::ErrorDiffusionAlg(int brightness)
{
	errorMap = 0;
	this->brightness = brightness;
}

ErrorDiffusionAlg::~ErrorDiffusionAlg()
{
	if(errorMap)
		delete errorMap;
}

void ErrorDiffusionAlg::touchDataLines(Map *imageMap, int start,int len,void *ptr)
{
	//create error map if needed
	if(errorMap == 0)
	{
		noErrorCounter[0] = 0;
		noErrorCounter[1] = 0;
		noErrorCounter[2] = 0;
		
		errorMap = new Map(imageMap->width, imageMap->height, sizeof(int)*3); // each color is held by 10 bits?
	}

	// make sure the correct lines are cached(map is not picky about caching before or past image)
	errorMap->cacheLines(start - FILTER_HEIGHT + 1, len + FILTER_HEIGHT - 1);
	
	// go through each point updating based on error map and filter
	for(int y = start; y < start+len; y++)
	{
		//on to the next line, subtract one from the no error counter
		for(int c = 0; c < 2; c++)
		{
			if(noErrorCounter[c])
				noErrorCounter[c] --;
		}
		

		int start_ey = max(0, FILTER_HEIGHT - 1 - y);
		
		for(int x = 0; x < imageMap->width; x++)
		{
			// update based on error map and filter matrix
			RGB *value = (RGB *)getMapCell(imageMap,x,y);

			for(int c = 0; c <= 2; c++)
			{
				int error = 0;

				// if there is an error
				if(noErrorCounter[c])
				{
					int start_ex = max(0, FILTER_WIDTH/2 - x);
					int end_ex = min(FILTER_WIDTH, imageMap->width - x + FILTER_WIDTH/2+1);
					
					for(int ey = start_ey; ey < FILTER_HEIGHT; ey++)
					{
						for(int ex = start_ex; ex < end_ex; ex++)
						{
							if(errorMatrix[ey][ex])
								error += ((int *)getMapCell(errorMap, ex + x - FILTER_WIDTH/2,ey + y - FILTER_HEIGHT + 1))[c] *
									errorMatrix[ey][ex];
						}
					}

					error = error / DIVIDER;
				}
				
				unsigned char *color = c == 0 ? &value->r : c == 1 ? &value->g : &value->b;
				
				int bigcolor = (int)*color + error;
				
				if(bigcolor >= brightness/2)
					*color = brightness;
				else
					*color = 0;

				error =  bigcolor - *color;

				// if there is an error we must reset the no error counter
				if(error != 0)
				{
					noErrorCounter[c] = FILTER_HEIGHT-1;
				}

				((int *)getMapCell(errorMap,x,y))[c] =error;
			}
		}

	}
	
}



