/*
Lexmark 1100 printer driver for Linux
Copyright (C) 1999   Tim Engler

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "math.h"
#include "Rect.h"

Rect::Rect()
{
	width = -1; // mark as uninitialized
}

Rect::Rect(int x, int y, int width, int height)
{
	this->x = x;
	this->y = y;
	this->width = width;
	this->height = height;
}


/* Sets dest to the minimum area which covers the current rectangle and "other". If a rectangles coordinates haven't
 * be defined yet, dest will always be set to the other rectangle's data.
 */
void Rect::getMax(Rect *dest, Rect *other)
{
	if(other->width == -1)
	{
		dest->x = x;
		dest->y = y;
		dest->width = width;
		dest->height = height;
		return;
	}
	if(width == -1)
	{
		dest->x = other->x;
		dest->y = other->y;
		dest->width = other->width;
		dest->height = other->height;
		return;
	}

	//save new x and y values in newX and newY incase destination rect is one of the ones we are comparing
	int newX = min(x,other->x);
	int newY = min(y,other->y);
	dest->width = max(x + width, other->x + other->width) - newX;
	dest->height = max(y + height,other->y + other->height) - newY;

	dest->x = newX;
	dest->y = newY;

}
